package com.javahonk.dao;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.transform.Transformers;
import org.hibernate.type.StandardBasicTypes;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.javahonk.model.TestModelForProcedure;
import com.javahonk.model.TestModelWithoutColumnMapping;

@Repository
@Transactional
public class StoredProcedureDaoHandler<T> implements StoredProcedureDao<T>{
	
	@Autowired
	private SessionFactory sessionFactory;
	
	@Override
	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true, propagation=Propagation.NOT_SUPPORTED)
	public List<T> getDataByNativeQuery(String eventDate) {
		
		Session session = sessionFactory.getCurrentSession();
		Query query = session.createSQLQuery(
				"{CALL OTC.dbo.TestStoreProcedure(:eventDate)}").addEntity(TestModelForProcedure.class);
		query.setParameter("eventDate", eventDate);
		List<T> dividendPayDate = query.list();
		return dividendPayDate;
	}
	
	@Override
	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true, propagation=Propagation.NOT_SUPPORTED)
	public List<T> getDataByNamedNativeQuery(String eventDate) {
		
		Session session = sessionFactory.getCurrentSession();
		Query query = session.getNamedQuery("callStoredProcedure");
			query.setParameter("eventDate", eventDate);
			List<T> dividendPayDate = query.list();
			return dividendPayDate;			
	}
	
	
	/* (non-Javadoc)
	 * @see com.javahonk.dao.StoreProcedureDao#getTestTableDataByProcedureExecute(java.lang.String)
	 * If you want to pass multiple parameter please use as below:
	 * "SET CHAINED OFF execute OTC.dbo.TestStoreProcedure :eventDate, :eventDate2, :eventDate3"
	 */
	@Override
	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true, propagation=Propagation.NOT_SUPPORTED)
	public List<T> getDataByDirectPureQuery(String eventDate) {
		
		Session session = sessionFactory.getCurrentSession();
		Query query = session.createSQLQuery(
				"execute OTC.dbo.TestStoreProcedure :eventDate")
				.addEntity(TestModelForProcedure.class);
		query.setParameter("eventDate", eventDate);
		List<T> dividendPayDate = query.list();
		return dividendPayDate;
	}
	
	@Override
	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true, propagation=Propagation.NOT_SUPPORTED)
	public List<T> getTestTableDataByUsingScalerTypes(String eventDate) {
		
		Session session = sessionFactory.getCurrentSession();
		Query query = session.createSQLQuery(
				"execute OTC.dbo.TestStoreProcedure :eventDate")
				.addScalar("test_id",StandardBasicTypes.INTEGER)
				.addScalar("smID", StandardBasicTypes.STRING)
				.addScalar("eventDate", StandardBasicTypes.DATE)
				.addScalar("createTime", StandardBasicTypes.DATE)
				.addScalar("updateTime", StandardBasicTypes.DATE)
				.setResultTransformer(Transformers.aliasToBean(TestModelWithoutColumnMapping.class));
		query.setParameter("eventDate", eventDate);
		List<T> dividendPayDate = query.list();
		return dividendPayDate;
	}

	@Override
	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true, propagation=Propagation.NOT_SUPPORTED)
	public List<T> getTestTableDataByUsingScalerWithoutTypes(String eventDate) {
		
		Session session = sessionFactory.getCurrentSession();
		Query query = session.createSQLQuery(
				"execute OTC.dbo.TestStoreProcedure :eventDate")
				.addScalar("test_id")
				.addScalar("smID")
				.addScalar("eventDate")
				.addScalar("createTime")
				.addScalar("updateTime")
				.setResultTransformer(Transformers.aliasToBean(TestModelWithoutColumnMapping.class));
		query.setParameter("eventDate", eventDate);
		List<T> dividendPayDate = query.list();
		return dividendPayDate;
	}
	

}
